<!doctype html>
<html>
  <head>
    <title>AudioTSM : Examples</title>
    <meta charset="UTF-8"> 
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" type="text/css" href="css/audiotsm.css">
  </head>
  <body>
    <header class="navbar">
      <div class="navbar-section">
        <a href="index.html" class="btn btn-link text-bold">AudioTSM</a>
        <a href="https://audiotsm.readthedocs.io/" class="btn btn-link">Documentation</a>
      </div>
      
      <div class="navbar-section">
        <a href="https://github.com/Muges/audiotsm" class="btn btn-primary icon-btn">
          <img src="images/github.svg" alt="" />
          <span>GitHub</span>
        </a>
      </div>
    </header>

    <section class="content content-width">
      <h2>AudioTSM examples</h2>

      <p>AudioTSM is a python library implementing various time-scale
      modification procedures, i.e. procedures that change the speed of an
      audio signal without changing its pitch. None of those methods are
      perfect, and the quality of the results depends on the audio signal and
      the parameters of the algorithm. This page gives an overview of the
      result obtained for each method with various inputs and at various
      speeds.</p>

      <p>The <a href="#credits">credits</a> for the audio files listed below
      are available at the end of this page.</p>
    </section>
    
    {% for speed in speeds %}
    <section class="content content-width">
      <h3>Speed = {{ speed }}</h3>

      <div class="examples">
        <table class="table table-striped table-hover">
          <thead>
            <tr>
              <th>Name</th>
              <th>Original</th>
              {% for _, method in methods %}
              <th>{{ method }}</th>
              {% endfor %}
            </tr>
          </thead>
          <tbody>
            {% for name, file in files[speed] | dictsort %}
            <tr>
              <td>{{ name }}</td>
              <td>
                <a href="{{ files.orig[name] }}" class="play-btn icon-btn btn-small">
                  <img src="images/play.svg" alt="" />
                  <span>Play</span>
                </a>
              </td>
              {% for method, _ in methods %}
              <td>
                <a href="{{ file[method] }}" class="play-btn icon-btn btn-small">
                  <img src="images/play.svg" alt="" />
                  <span>Play</span>
                </a>
              </td>
              {% endfor %}
            </tr>
            {% endfor %}
          </tbody>
        </table>
      </div>
    </section>
    {% endfor %}

    <div class="player">
      <audio controls id="player">
        <source id="playerSource">
      </audio>
    </div>

    <section class="content content-width">
      <h3 id="credits">Credits</h3>

      {{ credits }}
    </section>

    <script src="js/examples.js"></script>

    <footer>
      <div class="content-width">
        Designed with <a href="https://picturepan2.github.io/spectre/index.html">Spectre.css</a> and <a href="https://feathericons.com/">Feather</a>.
      </div>
    </footer>
  </body>
</html>
